#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "kp_core.h"
#include "kp_inference.h"
#include "helper_functions.h"

#include "my_KL730_sin_example.h"

static char _scpu_fw_path[128] = "user_data/kp_firmware.tar";
static char _model_file_path[128] = "user_data/YoloV5s_640_640_3/models_730.nef";
static char _image_file_path[128] = "user_data/3840x2160_48_3_10.bmp";
static int _loop = 10;

int main(int argc, char *argv[])
{
    kp_device_group_t device;
    kp_model_nef_descriptor_t model_desc;
    int ret;

	if (argc > 1) {
		memset(_scpu_fw_path, 0, 128);
		memcpy(_scpu_fw_path, argv[1], strlen(argv[1]));
	}
	
	if (argc > 2) {
		memset(_model_file_path, 0, 128);
		memcpy(_model_file_path, argv[2], strlen(argv[2]));
	}
	
	if (argc > 3) {
		memset(_image_file_path, 0, 128);
		memcpy(_image_file_path, argv[3], strlen(argv[3]));
	}
	fprintf(stderr, "%d\n", argc);
	fprintf(stderr, "%s\n", _scpu_fw_path);
	fprintf(stderr, "%s\n", _model_file_path);
	fprintf(stderr, "%s\n", _image_file_path);

    /******* connect the device *******/
    {
        int port_id = 0; // 0 for one device auto-search
        int error_code;

        // connect device
        device = kp_connect_devices(1, &port_id, &error_code);
        if (!device) {
            printf("connect device failed, port ID = '%d', error = '%d'\n", port_id, error_code);
            exit(0);
        }

        kp_set_timeout(device, 5000);
        printf("connect device ... OK\n");
    }

    /******* upload firmware to device *******/
    {
        ret = kp_load_firmware_from_file(device, _scpu_fw_path, NULL);
        if (ret != KP_SUCCESS) {
            printf("upload firmware failed, error = %d\n", ret);
            exit(0);
        }

        printf("upload firmware ... OK\n");
    }

    /******* upload model to device *******/
    {
        ret = kp_load_model_from_file(device, _model_file_path, &model_desc);
        if (ret != KP_SUCCESS) {
            printf("upload model failed, error = %d\n", ret);
            exit(0);
        }

        printf("upload model ... OK\n");
    }

    /******* prepare the image buffer read from file *******/
    // here convert a bmp file to RGB565 format buffer

    int img_width, img_height;
    char *img_buf = helper_bmp_file_to_raw_buffer(_image_file_path, &img_width, &img_height, KP_IMAGE_FORMAT_RGB565);

    if (!img_buf) {
        printf("read image file failed\n");
        exit(0);
    }

    printf("read image ... OK\n");
    printf("\nstarting inference loop %d times:\n", _loop);

    /******* prepare input and output header/buffers *******/
    my_KL730_sin_example_header_t input_header;
    my_KL730_sin_example_result_t output_result;

    input_header.header_stamp.job_id = MY_KL730_SIN_EXAMPLE_JOB_ID;
    input_header.header_stamp.total_image = 1;
    input_header.header_stamp.image_index = 0;
    input_header.img_width = img_width;
    input_header.img_height = img_height;

    int header_size = sizeof(my_KL730_sin_example_header_t);
    int image_size = img_width * img_height * 2; // RGB565
    int result_size = sizeof(my_KL730_sin_example_result_t);
    int recv_size = 0;

    /******* starting inference work *******/

    for (int i = 0; i < _loop; i++)
    {
        ret = kp_customized_inference_send(device, (void *)&input_header, header_size, (uint8_t *)img_buf, image_size);

        if (ret != KP_SUCCESS) {
            break;
        }

        ret = kp_customized_inference_receive(device, (void *)&output_result, result_size, &recv_size);

        if (ret != KP_SUCCESS) {
            break;
        }

        printf("\n[loop %d]\n", i + 1);
        helper_print_yolo_box_on_bmp((kp_yolo_result_t *)&output_result.yolo_result, _image_file_path);
    }

    printf("\n");

    if (ret != KP_SUCCESS) {
        printf("\ninference failed, error = %d\n", ret);
        return -1;
    }

    free(img_buf);
	kp_release_model_nef_descriptor(&model_desc);
    kp_disconnect_devices(device);

    return 0;
}